namespace EventAggregatorPattern
{
    using System;

    public abstract class BaseBackgroundTask : IBackgroundTask
    {
        private readonly IEventAggregator _eventAggregator;

        protected BaseBackgroundTask(IEventAggregator eventAggregator)
        {
            if (eventAggregator == null)
            {
                throw new ArgumentNullException("eventAggregator");
            }

            _eventAggregator = eventAggregator;
        }

        public bool IsRunning
        {
            get;
            private set;
        }

        protected IEventAggregator EventAggregator
        {
            get
            {
                return _eventAggregator;
            }
        }

        public void Start()
        {
            OnStart();
            IsRunning = true;
        }

        public void Stop()
        {
            OnStop();
            IsRunning = false;
        }

        protected abstract void OnStart();

        protected abstract void OnStop();

        protected SubscriptionToken Subscribe<TEvent, TEventArgs>(Action<TEventArgs> action) where TEvent : BaseEvent<TEventArgs> where TEventArgs : class
        {
            return EventAggregator.GetEvent<TEvent>().Subscribe(action, true);
        }

        protected void Unsubscribe<TEvent>(SubscriptionToken token) where TEvent : BaseEvent
        {
            EventAggregator.GetEvent<TEvent>().Unsubscribe(token);
        }
    }
}