namespace EventAggregatorPattern
{
    using System;

    public class PingServer : BaseBackgroundTask
    {
        private SubscriptionToken _storySubmitToken;

        public PingServer(IEventAggregator eventAggregator) : base(eventAggregator)
        {
        }

        protected override void OnStart()
        {
            if (!IsRunning)
            {
                _storySubmitToken = Subscribe<StorySubmitEvent, StorySubmitEventArgs>(StorySubmitted);
            }
        }

        protected override void OnStop()
        {
            if (IsRunning)
            {
                Unsubscribe<StorySubmitEvent>(_storySubmitToken);
            }
        }

        private void StorySubmitted(StorySubmitEventArgs eventArgs)
        {
            SendPing(eventArgs.Story);
        }

        private void SendPing(Story story)
        {
            //Feed Server ping goes here
            Console.WriteLine(story.ToString());
        }
    }
}