namespace EventAggregatorPattern
{
    using System;

    public class UserScoreService : BaseBackgroundTask
    {
        private SubscriptionToken _storySubmitToken;

        public UserScoreService(IEventAggregator eventAggregator) : base(eventAggregator)
        {
        }

        protected override void OnStart()
        {
            if (!IsRunning)
            {
                _storySubmitToken = Subscribe<StorySubmitEvent, StorySubmitEventArgs>(StorySubmitted);
            }
        }

        protected override void OnStop()
        {
            if (IsRunning)
            {
                Unsubscribe<StorySubmitEvent>(_storySubmitToken);
            }
        }

        private void StorySubmitted(StorySubmitEventArgs eventArgs)
        {
            IncreaseUserScore(eventArgs.Story.PostedBy);
        }

        private void IncreaseUserScore(User user)
        {
            // Increase user score goes here
            Console.WriteLine(user.ToString());
        }
    }
}