namespace EventAggregatorPattern
{
    using System.Web.Routing;
    using System.Web.Mvc;

    using Microsoft.Practices.ServiceLocation;
    using Microsoft.Practices.Unity;
    using Microsoft.Practices.Unity.ServiceLocatorAdapter;

    public static class Bootstrapper
    {
        static Bootstrapper()
        {
            ConfigureContainer();
        }

        public static void Run()
        {
            var tasks = ServiceLocator.Current.GetAllInstances<IBootstrapperTask>();

            foreach (IBootstrapperTask task in tasks)
            {
                task.Execute();
            }
        }

        private static void ConfigureContainer()
        {
            IUnityContainer container = new UnityContainer();

            container.RegisterType<IBackgroundTask, UserScoreService>("userScore", new ContainerControlledLifetimeManager())
                     .RegisterType<IBackgroundTask, SendTweet>("sendTweet", new ContainerControlledLifetimeManager())
                     .RegisterType<IBackgroundTask, PingStory>("pingStory", new ContainerControlledLifetimeManager())
                     .RegisterType<IBackgroundTask, PingServer>("pingServer", new ContainerControlledLifetimeManager())
                     .RegisterType<IEventAggregator, EventAggregator>(new ContainerControlledLifetimeManager())
                     .RegisterType<IControllerFactory, CommonServiceLocatorControllerFactory>(new ContainerControlledLifetimeManager())
                     .RegisterType<IBootstrapperTask, RegisterRoutes>("route", new ContainerControlledLifetimeManager(), new InjectionConstructor(new [] { RouteTable.Routes}))
                     .RegisterType<IBootstrapperTask, RegisterControllerFactory>("controllerFactory", new ContainerControlledLifetimeManager())
                     .RegisterType<IBootstrapperTask, StartBackgroundTasks>("startBackgroundTasks", new ContainerControlledLifetimeManager())
                     .RegisterType<IFormsAuthentication, FormsAuthenticationService>(new ContainerControlledLifetimeManager())
                     .RegisterType<IMembershipService, AccountMembershipService>(new InjectionConstructor())
                     .RegisterType<AccountController>()
                     .RegisterType<HomeController>();

            ServiceLocator.SetLocatorProvider(() => new UnityServiceLocator(container));
        }
    }
}